package org.gidea.scada.monitor.rabbit.config;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.rabbit.entity.MonitorMessage;
import org.gidea.scada.monitor.utils.MonitorUtils;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;
import org.springframework.amqp.core.Exchange;
import org.springframework.amqp.core.FanoutExchange;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static org.gidea.scada.monitor.utils.RabbitExchangeEnum.MonitorCommandClient;
import static org.gidea.scada.monitor.utils.RabbitExchangeEnum.MonitorCommandService;


@Log4j2
@Configuration
public class MonitorClientConfig {

    @Value("${org.gidea.monitor.client.name}")
    private String monitorName;
    @Resource
    private Channel channel;

    private Map<Integer, MonitorMessage> commandHistory = new HashMap<>();

    private Map<Integer, MonitorMessage> commandResult = new HashMap<>();


    @SneakyThrows
    @Bean(name = "commandClientExchange")
    public Exchange commandClientExchange(RabbitAdmin rabbitAdmin, SimpleRabbitListenerContainerFactory factory) {
        RabbitExchangeEnum exchangeEnum = MonitorCommandClient;
        FanoutExchange commandClientExchange = new FanoutExchange(exchangeEnum.exchangeName, false, false);
        rabbitAdmin.declareExchange(commandClientExchange);
        String queueName = MonitorUtils.getQueueName(exchangeEnum.exchangeName, monitorName);
        // 声明请求主队列
        Queue requestQueue = new Queue(queueName, false, false, false);
        rabbitAdmin.declareQueue(requestQueue);
        channel.queueBind(queueName, exchangeEnum.exchangeName, "");
        SimpleMessageListenerContainer container = factory.createListenerContainer();
        container.setQueueNames(queueName);
        container.setMessageListener(new ChannelAwareMessageListener() {
            @Override
            public void onMessage(Message message, Channel channel) {
                String messageStr = new String(message.getBody());
                MonitorMessage monitorMessage = JSONObject.parseObject(messageStr, MonitorMessage.class);
                MonitorMessage history = commandHistory.get(monitorMessage.getCmd());
                if (history != null) {
                    commandResult.put(monitorMessage.getCmd(), monitorMessage);
                    commandHistory.remove(monitorMessage.getCmd());
                }
                log.info("MonitorService接收数据：{}", messageStr);
            }
        });
        container.start();
        return commandClientExchange;
    }

    public void sendCommand(MonitorMessage message) throws IOException {
        channel.basicPublish(MonitorCommandService.exchangeName, "", null, message.toString().getBytes());
        commandHistory.put(message.getCmd(), message);
        if (commandResult.get(message.getCmd()) != null) {
            commandResult.remove(message.getCmd());
        }
    }

    public MonitorMessage getResult(int cmd) {
        return commandResult.get(cmd);
    }
}
