package org.gidea.scada.monitor.mqtt.listener;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.eclipse.paho.client.mqttv3.*;
import org.gidea.scada.monitor.mqtt.config.MonitorMqttConfig;
import org.gidea.scada.monitor.udp.entity.CjdMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;

@Log4j2
public class CjdServiceMqtt {
    private String topicName;

    private MonitorMqttConfig mqttConfig;

    private Channel channel;

    private MqttClient sampleClient;

    public CjdServiceMqtt(String topic, MonitorMqttConfig mqttConfig, Channel channel) {
        this.mqttConfig = mqttConfig;
        this.topicName = topic;
        this.channel = channel;
        // 创建 MQTT 客户端实例
        try {
            sampleClient = mqttConfig.mqttClient("MonitorMqttService_" + topicName);
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
    }

    public void publish(String topic, String message) {
        try {
            sampleClient.publish(topic, new MqttMessage(message.getBytes()));
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
    }

    public void listener() throws MqttException {
        // 订阅以 "mqtt." 开始的所有主题
        int qos = 2; // 设置服务质量级别
        sampleClient.subscribe(topicName, qos);
        // 设置回调以接收消息
        sampleClient.setCallback(new MqttCallback() {
            @Override
            public void connectionLost(Throwable cause) {
                System.out.println("Connection lost. Reason: " + cause);
            }

            /**
             * 接收到采集端返回的设备实时数据
             * @param topic name of the topic on the message was published to
             * @param message the actual message.
             * @throws Exception
             */
            @Override
            public void messageArrived(String topic, MqttMessage message) throws Exception {
                String msg = new String(message.getPayload());
                if (msg.endsWith("\n")) {
                    msg = msg.substring(0, msg.length() - 1);
                }
                String tempCrc = msg.substring(0, 4);
                String tempMessage = msg.substring(5);
                String crc = CrcUtils.getCrc(tempMessage.getBytes());
//                    // 数据校验
                if (crc.equalsIgnoreCase(tempCrc)) {
                    CjdMessage cjdMessage = JSONObject.parseObject(tempMessage, CjdMessage.class);
                    if (cjdMessage.getCode() == 0) {
                        log.info("【{}】主动上报：{}", topic, msg);
                    } else if (cjdMessage.getCode() == 1) {
                        log.info("【{}】响应指令：{}", topic, msg);
                    }
                    channel.basicPublish(RabbitExchangeEnum.MonitorCommandClient.exchangeName, "", null, JSONObject.toJSONString(cjdMessage).getBytes());
                }

            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken token) {
                System.out.println("Delivery complete for token: " + token);
            }
        });
    }
}
