package org.gidea.scada.monitor.mqtt.listener;


import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.eclipse.paho.client.mqttv3.*;
import org.gidea.scada.monitor.entity.ScadaTxsbVo;
import org.gidea.scada.monitor.mqtt.config.MonitorMqttConfig;
import org.gidea.scada.monitor.rabbit.entity.MonitorMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.gidea.scada.monitor.utils.EmptyUtil;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Component
public class MonitorMqttListener {

    Set<String> txsb = new HashSet<>();
    @Resource
    private Channel channel;
    @Value("${org.gidea.scada.txsb.uri}")
    private String txsbUri;
    @Resource
    private MonitorMqttConfig mqttConfig;
    @Resource
    private RestTemplateBuilder restTemplate;

    @PostConstruct
    @Scheduled(cron = "0/10 * * * * ?")
    public void queryTxsb() {
        RestTemplate template = restTemplate.build();
        List<ScadaTxsbVo> data = null;
        try {
            data = template.postForEntity(txsbUri, null, JSONObject.class).getBody().getObject("data", new TypeReference<List<ScadaTxsbVo>>() {
            });
        } catch (RestClientException e) {
            return;
        }
        List<String> bss = data.parallelStream().filter(o -> !txsb.contains(o.getSavetablename())).map(ScadaTxsbVo::getSavetablename).collect(Collectors.toList());
        if (!EmptyUtil.isEmpty(bss)) {
            txsb.addAll(bss);
            bss.parallelStream().forEach(bs -> {
                try {
                    new RealtimeMqtt("mqtt." + bs, mqttConfig, channel).listener();
                    new RealtimeMqtt("mqtt." + bs + ".alarm", mqttConfig, channel).listener();
                } catch (MqttException e) {
                    throw new RuntimeException(e);
                }
            });
        }
    }

    @Bean
    public MqttClient serverLister() throws MqttException {
        // 创建 MQTT 客户端实例
        MqttClient sampleClient = mqttConfig.mqttClient("MonitorMqttServer");
        // 订阅以 "mqtt." 开始的所有主题
        String wildcardTopic = "monitor.mqtt.server";
        int qos = 2; // 设置服务质量级别
        sampleClient.subscribe(wildcardTopic, qos);
        // 设置回调以接收消息
        sampleClient.setCallback(new MqttCallback() {
            @Override
            public void connectionLost(Throwable cause) {
                System.out.println("Connection lost. Reason: " + cause);
            }

            /**
             * 接收到采集端返回的响应数据
             * @param topic name of the topic on the message was published to
             * @param message the actual message.
             * @throws Exception
             */
            @Override
            public void messageArrived(String topic, MqttMessage message) throws Exception {
                String messageStr = new String(message.getPayload());
                if (messageStr.length() > 5) {
                    String str = messageStr.substring(5);
                    if ((messageStr.substring(0, 4).equalsIgnoreCase(CrcUtils.getCrc(str.getBytes())))) {
                        MonitorMessage monitorMessage = JSONObject.parseObject(str, MonitorMessage.class);
                        channel.basicPublish(RabbitExchangeEnum.MonitorCommandClient.exchangeName, "", null, JSONObject.toJSONString(monitorMessage).getBytes());
                        log.info("接收到采集端消息：{}", messageStr);
                        return;
                    }
                }
                log.info("接收到采集端不合法消息：{}", messageStr);
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken token) {
                System.out.println("Delivery complete for token: " + token);
            }
        });
        return sampleClient;
    }
}
