package org.gidea.scada.monitor.mqtt.listener;

import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.eclipse.paho.client.mqttv3.*;
import org.gidea.scada.monitor.mqtt.config.MonitorMqttConfig;
import org.gidea.scada.monitor.utils.MonitorUtils;

import java.io.IOException;


@Log4j2
public class RealtimeMqtt {

    private String topicName;

    private String exchangeName;

    private MonitorMqttConfig mqttConfig;

    private Channel channel;

    public RealtimeMqtt(String topic, MonitorMqttConfig mqttConfig, Channel channel) {
        this.mqttConfig = mqttConfig;
        this.topicName = topic;
        this.channel = channel;
        String deviceId = MonitorUtils.getDeviceIdByMqtt(topic);
        exchangeName = MonitorUtils.getDeviceExchangeName(deviceId);
        try {
            channel.exchangeDeclare(exchangeName, BuiltinExchangeType.FANOUT, false, false, null);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void listener() throws MqttException {
        // 创建 MQTT 客户端实例
        MqttClient sampleClient = mqttConfig.mqttClient("MonitorMqttRealtime_" + topicName);
        // 订阅以 "mqtt." 开始的所有主题
        int qos = 2; // 设置服务质量级别
        sampleClient.subscribe(topicName, qos);
        // 设置回调以接收消息
        sampleClient.setCallback(new MqttCallback() {
            @Override
            public void connectionLost(Throwable cause) {
                System.out.println("Connection lost. Reason: " + cause);
            }

            /**
             * 接收到采集端返回的设备实时数据
             * @param topic name of the topic on the message was published to
             * @param message the actual message.
             * @throws Exception
             */
            @Override
            public void messageArrived(String topic, MqttMessage message) throws Exception {
                String msg = new String(message.getPayload());
                log.info("【{}】接收到实时数据：{}", topic, msg);
                channel.basicPublish(exchangeName, "", null, message.getPayload());
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken token) {
                System.out.println("Delivery complete for token: " + token);
            }
        });
    }

}
