package org.gidea.scada.monitor.rabbit.config;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.rabbit.listener.CjdListenerHandler;
import org.gidea.scada.monitor.rabbit.listener.CommandListenerHandler;
import org.gidea.scada.monitor.udp.entity.CjdMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;
import org.springframework.amqp.core.Exchange;
import org.springframework.amqp.core.FanoutExchange;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;

import static org.gidea.scada.monitor.utils.RabbitExchangeEnum.*;

@Log4j2
@Configuration
public class MonitorRabbitMQConfig {
    @Resource
    private Channel channel;
    @Resource
    private CommandListenerHandler commandListenerHandler;
    @Resource
    private CjdListenerHandler cjdListenerHandler;

    @SneakyThrows
    @Bean(name = "commandServiceExchange")
    public Exchange commandServiceExchange(RabbitAdmin rabbitAdmin, SimpleRabbitListenerContainerFactory factory) {
        RabbitExchangeEnum exchangeEnum = MonitorCommandService;
        FanoutExchange commandServiceExchange = new FanoutExchange(exchangeEnum.exchangeName, false, false);
        rabbitAdmin.declareExchange(commandServiceExchange);

        // 声明请求主队列
        Queue requestQueue = new Queue(exchangeEnum.exchangeName, false, false, false);
        rabbitAdmin.declareQueue(requestQueue);
        channel.queueBind(exchangeEnum.exchangeName, exchangeEnum.exchangeName, "");
        SimpleMessageListenerContainer container = factory.createListenerContainer();
        container.setQueueNames(exchangeEnum.exchangeName);
        container.setMessageListener(new ChannelAwareMessageListener() {
            @Override
            public void onMessage(Message message, Channel channel) {
                String messageStr = new String(message.getBody());
                log.info("MonitorService接收数据：{}",messageStr);
                commandListenerHandler.handler(messageStr);
            }
        });
        container.start();
        return commandServiceExchange;
    }

    @SneakyThrows
    @Bean(name = "commandClientExchange")
    public Exchange commandClientExchange(RabbitAdmin rabbitAdmin) {
        FanoutExchange commandClientExchange = new FanoutExchange(MonitorCommandClient.exchangeName, false, false);
        rabbitAdmin.declareExchange(commandClientExchange);
        return commandClientExchange;
    }

}
