package org.gidea.scada.monitor.udp.config;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.rabbitmq.client.Channel;
import io.vertx.core.Vertx;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.datagram.DatagramSocket;
import io.vertx.core.net.SocketAddress;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.gidea.scada.monitor.mqtt.config.MonitorMqttConfig;
import org.gidea.scada.monitor.mqtt.listener.CjdServiceMqtt;
import org.gidea.scada.monitor.udp.entity.CjdMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.gidea.scada.monitor.utils.MonitorUtils;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.io.*;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;


@Slf4j
@Configuration
public class UdpServerConfig {
    @Value("${upd.server.port:23005}")
    private Integer listeningPort;


    @Value("${org.gidea.mqtt.uri}")
    private String url;

    @Value("${org.gidea.mqtt.username}")
    private String username;

    @Value("${org.gidea.mqtt.password}")
    private String password;
    @Resource
    private MonitorMqttConfig mqttConfig;
    @Resource
    private Channel channel;

    private Map<String, CjdMessage> ipMap = new LinkedHashMap<>();
    private String cjdFile = System.getProperty("user.dir") + "/config/cjd.json";

    private Map<String, CjdServiceMqtt> mqttMap = new HashMap<>();

    @Bean
    public DatagramSocket socket() {
        if (new File(cjdFile).exists()) {
            String line;
            try {
                // Read the JSON file
                BufferedReader reader = new BufferedReader(new FileReader(cjdFile));
                StringBuilder jsonData = new StringBuilder();
                while (true) {
                    if (!((line = reader.readLine()) != null)) {
                        break;
                    }
                    jsonData.append(line);
                }
                reader.close();
                Map<String, CjdMessage> data = JSONObject.parseObject(jsonData.toString(), new TypeReference<Map<String, CjdMessage>>() {
                });
                ipMap.putAll(data);
                for (CjdMessage cjdMessage : data.values()) {
                    CjdServiceMqtt serviceMqtt = new CjdServiceMqtt(MonitorUtils.getCjdServiceExchangeName(cjdMessage.getBscode()), mqttConfig, channel);
                    try {
                        serviceMqtt.listener();
                    } catch (MqttException e) {
                        throw new RuntimeException(e);
                    }
                    mqttMap.put(cjdMessage.getBscode(), serviceMqtt);
                }
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        Vertx vertx = Vertx.vertx();
        DatagramSocket socket = vertx.createDatagramSocket();
        socket.listen(23005, "0.0.0.0", result -> {
            if (result.succeeded()) {
                socket.handler(packet -> {
                    String message = packet.data().toString();
                    if (message.endsWith("\n")) {
                        message = message.substring(0, message.length() - 1);
                    }
                    String tempCrc = message.substring(0, 4);
                    String tempMessage = message.substring(5);
                    String crc = CrcUtils.getCrc(tempMessage.getBytes());
//                    // 数据校验
                    if (crc.equalsIgnoreCase(tempCrc)) {
                        CjdMessage cjdMessage = JSONObject.parseObject(tempMessage, CjdMessage.class);
                        switch (cjdMessage.getCode()) {
                            case 0:
                                CjdMessage handle0 = messageHandle0(cjdMessage);
                                SocketAddress sender = packet.sender();
                                String string = JSONObject.toJSONString(handle0);
                                String s = CrcUtils.getCrc(string.getBytes()) + "=" + string;
                                socket.send(Buffer.buffer(s), sender.port(), sender.host(), r -> {
                                    if (r.succeeded()) {
                                        System.out.println("【UDP服务】成功推送消息：" + s);
                                    } else {
                                        System.err.println("Failed to send reply: " + r.cause().getMessage());
                                    }
                                });
                                return;
                            case 1:
                                messageHandle1(cjdMessage);
                                return;
                            default:
                                messageHandle(cjdMessage);
                                return;
                        }
                    }
                    log.info("无法解析消息：{}", message);
                });
            } else {
                System.out.println("Listen failed" + result.cause());
            }
        });
        return socket;
    }

    private void messageHandle(CjdMessage message) {
        log.info("统一消息处理：{}", JSONObject.toJSONString(message));
    }

    private CjdMessage messageHandle0(CjdMessage message) {
        log.info("处理采集端主动推送数据：{}", JSONObject.toJSONString(message));
        CjdMessage cjdMessage = new CjdMessage(message);
        cjdMessage.setCode(1);
        if (10000 == message.getCmd()) {
            ipMap.put(message.getBscode(), message);
            File file = new File(cjdFile);
            if (!file.exists()) {
                try {
                    if (!file.getParentFile().exists()) {
                        file.getParentFile().mkdirs();
                    }
                    file.createNewFile();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
            String string = JSONObject.toJSONString(ipMap, true);
            // 将 JSON 字符串写入文件
            try (FileWriter fileWriter = new FileWriter(file)) {
                fileWriter.write(string);
                fileWriter.flush();
                fileWriter.close();
                System.out.println("JSON 数据已成功写入文件。");
            } catch (IOException e) {
                e.printStackTrace();
            }
            JSONObject mqtt = new JSONObject();
            mqtt.put("url", url);
            mqtt.put("username", username);
            mqtt.put("password", password);
            mqtt.put("service", RabbitExchangeEnum.MonitorCjdService.exchangeName);
            mqtt.put("client", RabbitExchangeEnum.MonitorCjdClient.exchangeName);
            cjdMessage.put("mqtt", mqtt);
            CjdServiceMqtt serviceMqtt = mqttMap.get(cjdMessage.getBscode());
            try {
                if (serviceMqtt == null) {
                    serviceMqtt = new CjdServiceMqtt(MonitorUtils.getCjdServiceExchangeName(cjdMessage.getBscode()), mqttConfig, channel);
                    serviceMqtt.listener();
                    mqttMap.put(cjdMessage.getBscode(), serviceMqtt);
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return cjdMessage;
    }

    private CjdMessage messageHandle1(CjdMessage message) {
        log.info("处理采集端响应结果数据：{}", JSONObject.toJSONString(message));
        return null;
    }


    public CjdServiceMqtt getMqtt(String bscode) {
        return mqttMap.get(bscode);
    }

    public CjdMessage getCjdIp(String cjdId) {
        return ipMap.get(cjdId);
    }
}

